/***************************************************************
  Sequencer.pde -- basic monophonic MIDI sequencer for Processing
  
  (C) 2011 Marc Groenewegen
  Hogeschool voor de Kunsten Utrecht

 The MIDI wrapper is needed for communicating with MIDI devices
 For a description see the header of MidiWrapper.java
****************************************************************/

MidiWrapper midi = new MidiWrapper();

float timeZero; // time at start of program
float now;

final int ENDOFSEQUENCE = -1; // illegal event to indicate loop end


float bpm=100; // quarter notes per minute
int qNote = (int)(60000/bpm); // duration of a quarter note in milliseconds


int index=0; // pointer into various arrays

// array of pitch (MIDI note number) values
int [] pitch = {
72,69,64,
72,69,64,
72,69,
71,67,64,
71,67,64,
71,67,
71,67,64,
71,67,64,
71,67,
69,66,62,
69,66,62,
69,66,ENDOFSEQUENCE};

float [] timeStamp = new float[pitch.length];
int [] velocity = new int[pitch.length];
int [] duration = new int[pitch.length];


void setup()
{
  size(500,500);
  frameRate(50);
  
  // Show available MIDI devices
  midi.listDevices();

  // Select a device for MIDI outputs
  midi.selectDevice(1);
 
  // Select drum channel
  midi.setMidiChannel(0);

  // Program change does not work for some unclear reason
  midi.programChange(0,35);

  // Put midi system in non-blocking mode
  midi.setBlocking(false);

  // timeZero indicates when the program started to run
  timeZero=millis();
 
  // fill some arrays with default values
  for(int i=0; i<pitch.length; i++)
  {
    timeStamp[i]=i * 0.5;
    duration[i]=200;

    // notes with higher pitch sound louder
    //velocity[i] = pitch[i]>68 ? 100 : 70;

    // slightly random velocity
    //velocity[i]=40+(int)random(60);

    // piano player falls asleep
    //velocity[i]=30 + (70*(pitch.length-i)/pitch.length);

    // constant velocity
    velocity[i]=80;
  } // for
} // setup()


void draw()
{
  background(255);
  ellipse(mouseX,mouseY,10,10); // give the graphic system something to do

  now=millis(); // get current time

  // is it time to play the next event?
  if( (now-timeZero) >= qNote*timeStamp[index])
  {
    if(pitch[index] != ENDOFSEQUENCE)
    {
      midi.playNote(pitch[index],velocity[index],duration[index]);
    }
    
    index++; // point to next event
    
    if(index==pitch.length) // all notes have been played
    {
      timeZero=millis(); // reset zero time reference
      index=0; // start again at first note
    } // if at end of array
  } // if time

} // draw()


void mousePressed() // set new BPM value upon mouse press
{
  bpm=40+160*mouseY/height; // BPM range 40..200
  qNote = (int)(60000/bpm);
  index=0; // start at beginning of loop
  timeZero=millis(); // reset time base to beginning of loop
  println("New bpm value: " + bpm + " new qNote value: " + qNote +  "msec");
} // mousePressed{}


void keyPressed()
{
  if(key == 'q') exit();
} // keyPressed{}

